/*   1 May 2000

   Interface from FORTRAN/GAMESS to C/system calls
   ===============================================
 
   Dynamic memory allocation for Unix systems - Jan 1990 - by Steve Elbert.
   The routines amount to a FORTRAN wrapper around C calls malloc and free.
   memory allocation: function invocation only: LOCMEM = MEMGET(NWORDS)
          NWORDS is the no. of 8-byte words of memory requested.
          The return value is the address of the allocated memory, if
          this is zero, the system doesn't have the memory available.
   memory release: subroutine call only: CALL MEMREL(LOCMEM)
          Deallocate all memory above the address LOCMEM.
          (deallocated memory is not returned to system)
   memory address: function invocation only: LOCX = LADDRS(X)
          Returns the absolute address of X, in bytes.  This is needed
          only if the FORTRAN library does not include the LOC function.

   Some machines have some timing routines added as well.
   One of these is important if MPI is being used to pass messages, in
   which case it allows DDI data servers to sleep when they are not in use,
   this is a subroutine call only: CALL NAPTIME(seconds)
   Other timing routines are just because of inadequate system libraries.

   Please note: there really shouldn't be anything here for a machine
   other than the MEMGET, MEMREL, and LADDRS routines, and maybe NAPTIME.
   One should refrain from adding anything else to this file at all.
*/

/*--------------- Compaq (aka Digital) --------------*/

#ifdef COMPAQ

int memget_(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }
#endif
 
/*----------------- Cray (all types) ----------------*/

#ifdef CRAY

int MEMGET(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void MEMREL(locmem) int *locmem;
   { free(*locmem); }
#endif

/*-------------- Fujitsu VPP and AP ---------------*/

#ifdef FUJITSU

int memget_(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel_(locmem) int *locmem;
   { free(*locmem); }

int laddrs_(arg) int arg;
   { return(arg); }
#endif

/*----------------- Hewlett-Packard -----------------*/
 
#ifdef HP9000

int memget(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel(locmem) int *locmem;
   { free(*locmem); }
/*
      Special CPU timing routine for Hewlett-Packard
      Written by Shinichi Yamada,
      YHP (Yokogawa Hewlett-Packard), Tokyo, Japan.  in Nov 1992.
      wall timing modified by Dave Mullally (Convex) in Jun 1994.
      Francois Rotzinger (Lausanne) renamed etime --> hptme Jun 1995.
 
      This is a clone of the usual Unix ETIME function,
      except usage is as a subroutine call,
        REAL XTIME(2)
        CALL HPTME(XTIME)
 
      HPTME will return two values:
        xtime(1)      <==== user cpu time    ( sec )
        xtime(2)      <==== system cpu time  ( sec )
      add the latter two to get actual CPU time.
*/

#include <sys/types.h>
#include <sys/times.h>
#include <unistd.h>
#include <sys/time.h>
void hptme(hptme_val)
float *hptme_val;
{
struct tms      buf;
static float  clock_val=0.0;
if ( clock_val == 0.0 ) {
  clock_val = (float)sysconf(_SC_CLK_TCK);
}
times(&buf);
*hptme_val = (float)(buf.tms_utime);
*hptme_val = *hptme_val/clock_val;
hptme_val++;
*hptme_val = (float)(buf.tms_stime);
*hptme_val = *hptme_val/clock_val;
return;
}
void walltim(hptme1)
double  *hptme1;
{
struct timeval   time_sec;
struct timezone  tzone;
float  clock_val;
gettimeofday(&time_sec,&tzone);
*hptme1 = (double)time_sec.tv_sec;
return;
}
#endif

/*----------- International Business Machines -----------*/
/*  code is for 32 bit Power, Power2, PowerPC systems,   */
/*  or for 64 bit Power3 workstations, or for 32 bit SP. */
/*  Note that all SP must be 32 bits until IBM           */
/*  gets around to releasing a 64 bit MPI library.       */
 
#ifdef IBM32

int memget(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel(locmem) int *locmem;
   { free(*locmem); }

int laddrs(arg) int arg;
   { return(arg); }
#endif

#ifdef IBMSP

int memget(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel(locmem) int *locmem;
   { free(*locmem); }

int laddrs(arg) int arg;
   { return(arg); }

#include <unistd.h>
void naptime(time)
useconds_t *time;
{
   int istat;
   istat=sleep(*time);
}
#endif

#ifdef IBM64

long memget(nwords) long *nwords;
   { long nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel(locmem) long *locmem;
   { free(*locmem); }

long laddrs(arg) int arg;
   { return(arg); }
#endif

/*----------------- PC running Linux -----------------*/

#ifdef gnu

int memget_(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel_(locmem) int *locmem;
   { free(*locmem); }

int laddrs_(arg) int arg;
   { return(arg); }


#include <unistd.h>
void naptime_(xtime)
     unsigned int *xtime;
{
  usleep(*xtime);
}

/*
     etime - return elapsed execution time
     double precision function etime (tarray)
     real tarray(2)
     written by Klaus-Peter Gulden, January 3, 1994
*/
#include <time.h>
#include <sys/times.h>

double xetime_(float *a)
{
  double        elapsed;
  struct tms    buf;
/*      CLK_TCK counts hundredths of seconds   */
  elapsed= (float) times(&buf)/ (float) CLK_TCK;
  a[0]=(float)(buf.tms_utime + buf.tms_cutime)/CLK_TCK;
  a[1]=(float)(buf.tms_stime + buf.tms_cstime)/CLK_TCK;
  return(elapsed);
}

/*
     fdate - return date and time in an ASCII string
     subroutine fdate(string)
     character*24 string

     returns the current date and time as a 24 character string in the
     format of ctime(3).  Neither `newline' nor NULL will be included.
     by Istvan Cserny, June 21, 1995
*/

void xfdate_(char *cht, int cht_len)
{
    static time_t t;
    time(&t);
    s_copy(cht,ctime(&t),24L,24L);
    return;
}
#endif
 
/*----------------- NEC SX series -----------------*/

#ifdef NECSX

void memget_(nwords,sxaddr)
 long long *nwords, **sxaddr ;
   {char *malloc(),*card;
    unsigned nbytes;
    nbytes = (unsigned) ((*nwords+2)*8);
    card =  malloc (nbytes);
    *sxaddr = (long long *) card; }

void memrel_(locmem)
 long long *locmem;
 {void free();
  char* ptrmem;
  ptrmem = (char *)locmem;
  free(ptrmem); }

void laddrx_(arg,addrarg)
long long *arg, **addrarg;
   { *addrarg = arg ; }
#endif

/*----------------- SGI -----------------*/

#ifdef SGI

int memget_(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel_(locmem) int *locmem;
   { free(*locmem); }

#include <unistd.h>
void naptime_(int *time)
{
   sginap((long) *time);
}
#endif

/*----------------- SUN -----------------*/

#ifdef SUN

int memget_(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel_(locmem) int *locmem;
   { free(*locmem); }
#endif
